@@ -1,8 +1,9 @@
 import new, sys, os
+
 import numpy
 from numpy import (
-    array, arange, empty, zeros, int32, uint16, complex_, float64, rec,
-    copy, ones_like, where, alltrue,
+    array, arange, empty, zeros, int32, int64, uint16, complex_, float64, rec,
+    copy, ones_like, where, alltrue, linspace,
     sum, prod, sqrt, fmod,
     sin, cos, tan, arcsin, arccos, arctan, arctan2,
     sinh, cosh, tanh, arcsinh, arccosh, arctanh,
@@ -22,6 +23,10 @@ double = numpy.double
 minimum_numpy_version = "1.2"
 
 class test_numexpr(TestCase):
+
+    def setUp(self):
+        numexpr.set_num_threads(self.nthreads)
+
     def test_simple(self):
         ex = 2.0 * E.a + 3.0 * E.b * E.c
         sig = [('a', double), ('b', double), ('c', double)]
@@ -37,14 +42,14 @@ class test_numexpr(TestCase):
 
     def test_simple_expr(self):
         func = NumExpr(E.a)
-        x = arange(1e5)
+        x = arange(1e6)
         y = func(x)
         assert_array_equal(x, y)
 
     def test_rational_expr(self):
         func = NumExpr((E.a + 2.0*E.b) / (1 + E.a + 4*E.b*E.b))
-        a = arange(1e5)
-        b = arange(1e5) * 0.1
+        a = arange(1e6)
+        b = arange(1e6) * 0.1
         x = (a + 2*b) / (1 + a + 4*b*b)
         y = func(a, b)
         assert_array_almost_equal(x, y)
@@ -114,6 +119,16 @@ class test_numexpr(TestCase):
                     [('mul_ddd', 'r0', 'r1[x]', 'r1[x]'),
                      ('add_ddd', 'r0', 'r0', 'c2[2.0]')])
 
+
+class test_numexpr1(test_numexpr):
+    """Testing with 1 thread"""
+    nthreads = 1
+
+class test_numexpr2(test_numexpr):
+    """Testing with 2 threads"""
+    nthreads = 2
+
+
 class test_evaluate(TestCase):
     def test_simple(self):
         a = array([1., 2., 3.])
@@ -128,13 +143,13 @@ class test_evaluate(TestCase):
         assert_array_equal(x, y)
 
     def test_simple_expr(self):
-        x = arange(1e5)
+        x = arange(1e6)
         y = evaluate("x")
         assert_array_equal(x, y)
 
     def test_rational_expr(self):
-        a = arange(1e5)
-        b = arange(1e5) * 0.1
+        a = arange(1e6)
+        b = arange(1e6) * 0.1
         x = (a + 2*b) / (1 + a + 4*b*b)
         y = evaluate("(a + 2*b) / (1 + a + 4*b*b)")
         assert_array_almost_equal(x, y)
@@ -153,7 +168,6 @@ class test_evaluate(TestCase):
         y = evaluate("sin(complex(a, b)).real + z.imag")
         assert_array_almost_equal(x, y)
 
-
     def test_complex_strides(self):
         a = arange(100).reshape(10,10)[::2]
         b = arange(50).reshape(5,10)
@@ -166,7 +180,6 @@ class test_evaluate(TestCase):
         assert_array_equal(evaluate("c1"), c1)
         assert_array_equal(evaluate("a0+c1"), a0+c1)
 
-
     def test_broadcasting(self):
         a = arange(100).reshape(10,10)[::2]
         c = arange(10)
@@ -198,10 +211,27 @@ class test_evaluate(TestCase):
         else:
             self.fail()
 
+    def test_changing_nthreads_inc(self):
+        a = linspace(-1, 1, 1e6)
+        b = ((.25*a + .75)*a - 1.5)*a - 2
+        for nthreads in range(1,7):
+            numexpr.set_num_threads(nthreads)
+            c = evaluate("((.25*a + .75)*a - 1.5)*a - 2")
+            assert_array_almost_equal(b, c)
+
+    def test_changing_nthreads_dec(self):
+        a = linspace(-1, 1, 1e6)
+        b = ((.25*a + .75)*a - 1.5)*a - 2
+        for nthreads in range(6, 0):
+            numexpr.set_num_threads(nthreads)
+            c = evaluate("((.25*a + .75)*a - 1.5)*a - 2")
+            assert_array_almost_equal(b, c)
+
 
 tests = [
 ('MISC', ['b*c+d*e',
           '2*a+3*b',
+          '-a',
           'sinh(a)',
           '2*a + (cos(3)+5)*sinh(cos(b))',
           '2*a + arctan2(a, b)',
@@ -278,15 +308,12 @@ def equal(a, b, exact):
 
 class Skip(Exception): pass
 
-class test_expressions(TestCase):
-    pass
-
-def generate_test_expressions():
+def test_expressions():
     test_no = [0]
     def make_test_method(a, a2, b, c, d, e, x, expr,
                          test_scalar, dtype, optimization, exact):
         this_locals = locals()
-        def method(self):
+        def method():
             npval = eval(expr, globals(), this_locals)
             try:
                 neval = evaluate(expr, local_dict=this_locals,
@@ -303,10 +330,12 @@ def generate_test_expressions():
             except:
                 print('numexpr error for expression %r' % (expr,))
                 raise
+        method.description = ('test_expressions(%s, test_scalar=%r, '
+                              'dtype=%r, optimization=%r, exact=%r)') \
+                    % (expr, test_scalar, dtype.__name__, optimization, exact)
         test_no[0] += 1
-        name = 'test_%04d' % (test_no[0],)
-        setattr(test_expressions, name,
-                new.instancemethod(method, None, test_expressions))
+        method.__name__ = 'test_%04d' % (test_no[0],)
+        return method
     x = None
     for test_scalar in [0,1,2]:
         for dtype in [int, long, numpy.float32, double, complex]:
@@ -339,11 +368,17 @@ def generate_test_expressions():
                         if (dtype in (int, long) and test_scalar and
                             expr == '(a+1) ** -1'):
                             continue
-                        make_test_method(a, a2, b, c, d, e, x,
-                                         expr, test_scalar, dtype,
-                                         optimization, exact)
+                        m = make_test_method(a, a2, b, c, d, e, x,
+                                             expr, test_scalar, dtype,
+                                             optimization, exact)
+                        yield m,
 
-generate_test_expressions()
+class test_int64(TestCase):
+    def test_neg(self):
+        a = array([2**31-1, 2**31, 2**32, 2**63-1], dtype=int64)
+        res = evaluate('-a')
+        assert_array_equal(res, [1-2**31, -(2**31), -(2**32), 1-2**63])
+        self.assertEqual(res.dtype.name, 'int64')
 
 class test_int32_int64(TestCase):
     def test_small_long(self):
@@ -542,6 +577,7 @@ def print_versions():
     print "VML available?     %s" % use_vml
     if use_vml:
         print "VML/MKL version:   %s" % numexpr.get_vml_version()
+    print 'Detected cores:    %s' % numexpr.ncores
     print '-=' * 38
 
 
@@ -552,6 +588,7 @@ def test():
 
     print_versions()
     unittest.TextTestRunner().run(suite())
+test.__test__ = False
 
 
 def suite():
@@ -560,10 +597,19 @@ def suite():
     theSuite = unittest.TestSuite()
     niter = 1
 
+    class TestExpressions(TestCase):
+        pass
+    for m, in test_expressions():
+        def method(self):
+            return m()
+        setattr(TestExpressions, m.__name__,
+                new.instancemethod(method, None, TestExpressions))
+
     for n in range(niter):
-        theSuite.addTest(unittest.makeSuite(test_numexpr))
+        theSuite.addTest(unittest.makeSuite(test_numexpr1))
+        theSuite.addTest(unittest.makeSuite(test_numexpr2))
         theSuite.addTest(unittest.makeSuite(test_evaluate))
-        theSuite.addTest(unittest.makeSuite(test_expressions))
+        theSuite.addTest(unittest.makeSuite(TestExpressions))
         theSuite.addTest(unittest.makeSuite(test_int32_int64))
         theSuite.addTest(unittest.makeSuite(test_uint32_int64))
         theSuite.addTest(unittest.makeSuite(test_strings))
